var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';
import PropTypes from 'prop-types';
import { merge } from 'lodash-es';
import { TransitionMotion, spring } from 'react-motion';
import Nivo, { defaultTheme } from '../../../Nivo';
import { marginPropType, motionPropTypes } from '../../../props';
import { getColorsGenerator, getInheritedColorGenerator } from '../../../lib/colorUtils';
import { generateGroupedBars, generateStackedBars } from '../../../lib/charts/bar';
import Container from '../Container';
import SvgWrapper from '../SvgWrapper';
import Axes from '../../axes/Axes';
import Grid from '../../axes/Grid';
import BarItem from './BarItem';
import BarItemLabel from './BarItemLabel';

var Bar = function Bar(_ref) {
    var data = _ref.data,
        groupMode = _ref.groupMode,
        _margin = _ref.margin,
        _width = _ref.width,
        _height = _ref.height,
        xPadding = _ref.xPadding,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        enableLabels = _ref.enableLabels,
        _labelsLinkColor = _ref.labelsLinkColor,
        _labelsTextColor = _ref.labelsTextColor,
        _theme = _ref.theme,
        colors = _ref.colors,
        colorBy = _ref.colorBy,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var margin = Object.assign({}, Nivo.defaults.margin, _margin);
    var width = _width - margin.left - margin.right;
    var height = _height - margin.top - margin.bottom;

    var theme = merge({}, defaultTheme, _theme);
    var color = getColorsGenerator(colors, colorBy);
    var labelsLinkColor = getInheritedColorGenerator(_labelsLinkColor, 'axis.tickColor');
    var labelsTextColor = getInheritedColorGenerator(_labelsTextColor, 'axis.textColor');

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var result = void 0;
    if (groupMode === 'grouped') {
        result = generateGroupedBars(data, width, height, color, {
            xPadding: xPadding
        });
    } else if (groupMode === 'stacked') {
        result = generateStackedBars(data, width, height, color, {
            xPadding: xPadding
        });
    }

    return React.createElement(
        Container,
        { isInteractive: isInteractive },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;

            var bars = void 0;
            if (animate === true) {
                bars = React.createElement(
                    TransitionMotion,
                    {
                        styles: result.bars.map(function (bar) {
                            return {
                                key: bar.key,
                                data: bar,
                                style: {
                                    x: spring(bar.x, motionProps),
                                    y: spring(bar.y, motionProps),
                                    width: spring(bar.width, motionProps),
                                    height: spring(bar.height, motionProps)
                                }
                            };
                        })
                    },
                    function (interpolatedStyles) {
                        return React.createElement(
                            'g',
                            null,
                            interpolatedStyles.map(function (_ref3) {
                                var key = _ref3.key,
                                    style = _ref3.style,
                                    data = _ref3.data;
                                return React.createElement(BarItem, _extends({
                                    key: key
                                }, data, style, {
                                    showTooltip: showTooltip,
                                    hideTooltip: hideTooltip
                                }));
                            })
                        );
                    }
                );
            } else {
                bars = result.bars.map(function (d) {
                    return React.createElement(BarItem, _extends({
                        key: d.key
                    }, d, {
                        showTooltip: showTooltip,
                        hideTooltip: hideTooltip
                    }));
                });
            }

            return React.createElement(
                SvgWrapper,
                { width: _width, height: _height, margin: margin },
                React.createElement(Grid, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? result.xScale : null,
                    yScale: enableGridY ? result.yScale : null
                }, motionProps)),
                React.createElement(Axes, _extends({
                    xScale: result.xScale,
                    yScale: result.yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                bars,
                enableLabels && result.bars.map(function (d) {
                    return React.createElement(BarItemLabel, _extends({}, d, {
                        key: d.key,
                        linkColor: labelsLinkColor(d, theme),
                        textColor: labelsTextColor(d, theme)
                    }));
                })
            );
        }
    );
};

Bar.propTypes = _extends({
    // data
    data: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string.isRequired,
        data: PropTypes.arrayOf(PropTypes.shape({
            x: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired,
            y: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired
        })).isRequired
    })).isRequired,

    groupMode: PropTypes.oneOf(['stacked', 'grouped']).isRequired,

    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    margin: marginPropType,
    xPadding: PropTypes.number.isRequired,

    // axes & grid
    axisTop: PropTypes.object,
    axisRight: PropTypes.object,
    axisBottom: PropTypes.object,
    axisLeft: PropTypes.object,
    enableGridX: PropTypes.bool.isRequired,
    enableGridY: PropTypes.bool.isRequired,

    // labels
    enableLabels: PropTypes.bool.isRequired,
    labelsTextColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,

    // interactions
    onClick: PropTypes.func,

    // theming
    theme: PropTypes.object.isRequired,
    colors: PropTypes.any.isRequired,
    colorBy: PropTypes.oneOfType([PropTypes.string, PropTypes.func])

}, motionPropTypes, {

    isInteractive: PropTypes.bool
});

Bar.defaultProps = {
    margin: Nivo.defaults.margin,
    groupMode: 'stacked',
    xPadding: 0.1,

    // axes & grid
    axisBottom: {},
    axisLeft: {},
    enableGridX: false,
    enableGridY: true,

    // labels
    enableLabels: true,
    labelsLinkColor: 'theme',
    labelsTextColor: 'theme',

    // theming
    theme: {},
    colors: 'nivo',
    colorBy: 'serie.id',

    // motion
    animate: true,
    motionStiffness: Nivo.defaults.motionStiffness,
    motionDamping: Nivo.defaults.motionDamping
};

export default Bar;