var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';
import PropTypes from 'prop-types';
import { merge, isEqual, min, max, range } from 'lodash-es';
import { stack as d3Stack, area as d3Area } from 'd3-shape';
import { scaleLinear, scalePoint } from 'd3-scale';
import compose from 'recompose/compose';
import pure from 'recompose/pure';
import withPropsOnChange from 'recompose/withPropsOnChange';
import defaultProps from 'recompose/defaultProps';
import Nivo, { defaultTheme } from '../../../Nivo';
import { marginPropType, motionPropTypes, areaCurvePropType, curveFromProp, stackOrderPropType, stackOrderFromProp, stackOffsetPropType, stackOffsetFromProp } from '../../../props';
import { getColorRange } from '../../../lib/colorUtils';
import SvgWrapper from '../SvgWrapper';
import Container from '../Container';
import Axes from '../../axes/Axes';
import Grid from '../../axes/Grid';
import StreamLayers from './StreamLayers';

var stackMin = function stackMin(layers) {
    return min(layers.reduce(function (acc, layer) {
        return [].concat(acc, layer.map(function (d) {
            return d[0];
        }));
    }, []));
};
var stackMax = function stackMax(layers) {
    return max(layers.reduce(function (acc, layer) {
        return [].concat(acc, layer.map(function (d) {
            return d[1];
        }));
    }, []));
};

var Stream = function Stream(_ref) {
    var data = _ref.data,
        keys = _ref.keys,
        order = _ref.order,
        offsetType = _ref.offsetType,
        curve = _ref.curve,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        fullWidth = _ref.fullWidth,
        fullHeight = _ref.fullHeight,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        theme = _ref.theme,
        color = _ref.color,
        fillOpacity = _ref.fillOpacity,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var stack = d3Stack().keys(keys).offset(stackOffsetFromProp(offsetType)).order(stackOrderFromProp(order));

    var layers = stack(data);

    var minValue = stackMin(layers);
    var maxValue = stackMax(layers);

    var xScale = scalePoint().domain(range(data.length)).range([0, width]);
    var yScale = scaleLinear().domain([minValue, maxValue]).range([height, 0]);

    var area = d3Area().x(function (d, i) {
        return xScale(i);
    }).y0(function (d) {
        return yScale(d[0]);
    }).y1(function (d) {
        return yScale(d[1]);
    }).curve(curveFromProp(curve));

    var enhancedLayers = layers.map(function (layer, i) {
        return {
            id: keys[i],
            layer: layer,
            path: area(layer),
            color: color(i)
        };
    });

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    return React.createElement(
        Container,
        { isInteractive: isInteractive },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;
            return React.createElement(
                SvgWrapper,
                { width: fullWidth, height: fullHeight, margin: margin },
                React.createElement(Grid, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? xScale : null,
                    yScale: enableGridY ? yScale : null
                }, motionProps)),
                React.createElement(StreamLayers, _extends({
                    layers: enhancedLayers,
                    area: area,
                    fillOpacity: fillOpacity,
                    showTooltip: showTooltip,
                    hideTooltip: hideTooltip
                }, motionProps)),
                React.createElement(Axes, _extends({
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps))
            );
        }
    );
};

Stream.propTypes = _extends({
    // data
    data: PropTypes.arrayOf(PropTypes.object).isRequired,
    keys: PropTypes.array.isRequired,

    order: stackOrderPropType.isRequired,
    offsetType: stackOffsetPropType.isRequired,
    curve: areaCurvePropType.isRequired,

    // dimensions
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    margin: marginPropType,

    // axes & grid
    axisTop: PropTypes.object,
    axisRight: PropTypes.object,
    axisBottom: PropTypes.object,
    axisLeft: PropTypes.object,
    enableGridX: PropTypes.bool.isRequired,
    enableGridY: PropTypes.bool.isRequired,

    // theming
    theme: PropTypes.object.isRequired,
    colors: PropTypes.any.isRequired,
    fillOpacity: PropTypes.number.isRequired,
    color: PropTypes.func.isRequired

}, motionPropTypes, {

    isInteractive: PropTypes.bool
});

export var StreamDefaultProps = {
    order: 'none',
    offsetType: 'wiggle',
    curve: 'catmullRom',

    // dimensions
    margin: Nivo.defaults.margin,

    // axes & grid
    axisBottom: {},
    enableGridX: true,
    enableGridY: false,

    // theming
    theme: {},
    colors: 'nivo',
    fillOpacity: 1,

    // motion
    animate: true,
    motionStiffness: Nivo.defaults.motionStiffness,
    motionDamping: Nivo.defaults.motionDamping
};

var enhance = compose(defaultProps(StreamDefaultProps), withPropsOnChange(['theme'], function (_ref3) {
    var theme = _ref3.theme;
    return { theme: merge({}, defaultTheme, theme) };
}), withPropsOnChange(['colors'], function (_ref4) {
    var colors = _ref4.colors,
        colorBy = _ref4.colorBy;
    return {
        color: getColorRange(colors)
    };
}), withPropsOnChange(function (props, nextProps) {
    return props.data !== nextProps.data || props.stacked !== nextProps.stacked || props.width !== nextProps.width || props.height !== nextProps.height || !isEqual(props.margin, nextProps.margin);
}, function (_ref5) {
    var data = _ref5.data,
        fullWidth = _ref5.width,
        fullHeight = _ref5.height,
        _margin = _ref5.margin;

    var margin = Object.assign({}, Nivo.defaults.margin, _margin);
    var width = fullWidth - margin.left - margin.right;
    var height = fullHeight - margin.top - margin.bottom;

    return {
        margin: margin,
        width: width,
        height: height,
        fullWidth: fullWidth,
        fullHeight: fullHeight
    };
}), pure);

export default enhance(Stream);